<?php

namespace HCM\FPA\Calculos\Apolo;

require_once __DIR__.'/bootstrapConsumer.php';
error_reporting(E_ERROR | E_PARSE);
ini_set("memory_limit","2048M");

use HCM\Geral\Logic\ImportacaoXmlEsocial;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\HCMParameters;
use HCM\Util\Mail\Mail;
use Zeedhi\Framework\Session\Session;
use Teknisa\Libs\Util\SMTP;

class MigracaoXmlConsumer extends \Zeedhi\Queue\Consumer {

    const QUEUE_NAME_MIGRACAO = "hcm_MigracaoXml_";

	protected static $queueName = '/queue/';
	
    public $fpaCalculoConsumer;
    
    private $nrcalculoconsumer;
    private $dsemailoper;
    private $cdoperador;
    private $dtmescompetencia;
    private $nrorg;
    private $tp_processamento;
    private $tipo_calculo;
    private $nrFormulario;
    private $pid;
    private $corpoEmail;
    private $mensagens = array();
    
    public function setQueueName($queueName, $pid) {
        self::$queueName .= $queueName;
        $this->pid = $pid;
    }
    
	/**
     * @param string $message
     * @return void
     */
    public function consume($message) {        
        // Tempo de execucao deve ser ilimitado.
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
            
        $session = new Session();
        $this->hcmParameters = new HCMParameters($session, FPA::getTempDirectory(), null, null);
        
        $this->entityManager = \Setforms2\Shared\Registry::getEntityManager();
            
    	try {
            echo ">>>>>>>>>> Iniciando Processo <<<<<<<<<<".PHP_EOL;
            echo 'Pid = '.$this->pid.PHP_EOL;
            
            $consumerParams = json_decode($message, true);
            
            $migracaoParams  = $consumerParams['PARAMS'][0];
            $registros = $migracaoParams['REGISTROS'];
            
            $this->setParameters($migracaoParams);
            
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            
            $logic = new ImportacaoXmlEsocial($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao(), $this->hcmParameters->getCdoperador(), $this->hcmParameters->getDtCompetencia());
            
            foreach($registros as $nrmigracaoxml){
                $migracaoxml = $this->entityManager->getRepository(Repositories::ESO_MIGRACAOXML)->findOneBy(array('nrorg' => $nrorg, 'nrmigracaoxml' => $nrmigracaoxml));
                $logic->importarMigracaoXml($migracaoxml);
            }
    
            // Define parametros do operador.
            $ambiente = $this->showAmbiente();
            
            //Envia email para o operador informando o término do calculo.
            $this->notificaMigracao($ambiente);
            return array(true, $this->pid, array_unique($this->pid));
            
        } catch (\Exception $e) {
            echo "Erro: ".$e->getMessage();
            $this->kill($this->pid);
            die();
        }
    }
    
    public function notificaMigracao($ambiente){
        $cdoperador = FPA::getCdOperador();
        $operador = self::retornaOperadorLogado($cdoperador);

        $emailOperadorLogado = $operador->getDsemailoper();
        $nomeOperadorLogado = $operador->getNmoperador();
        $nome_destinatario = $nomeOperadorLogado;
        
        $assunto = "Assistente de Migração de Dados - Importação XML";
        
        $corpo_msg = 
            $nomeOperadorLogado.
            "<br> ".$ambiente." <br>".
            "<br> Importação de itens finalizada. Verifique o log na tela 'Assistente de Migração de Dados.";
        
        $mail = $this->hcmParameters->returnParamsSmtp(true);
 
        if(!$mail){
            return;
        }
        
        try {
            $return = $mail->newEmail(
                $emailOperadorLogado,  //De/From
                $corpo_msg,            //texto
                $assunto,              //assunto    
                $nomeOperadorLogado,   //nome operador
                $nomeOperadorLogado,   //nome from
                $emailOperadorLogado   //from
            );
            if($return === false){
                echo "Erro ao enviar email <br>";
            }
        }catch (Exception $e) {
            $contFail++;
        }
    }

    public function retornaOperadorLogado($cdoperador){
        $operador = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::OPERADOR)->findOneBy(array('cdoperador' => $cdoperador, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));
        return $operador;
    }

    private function showAmbiente() {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetencia = $this->hcmParameters->getDtCompetencia();
        $cdoperador = $this->hcmParameters->getCdoperador();
        $dataExecucao = DateUtil::getDataAtual();

        $ambiente = "Org.: $nrorg\n Competencia: $dtmescompetencia\n Operador: $cdoperador\n Data Execucao: $dataExecucao";
        
        echo $ambiente."<br>";
        echo "<br>"."---------------"."<br>";
        
        return $ambiente;
    }

    private function setParameters($consumerParams) {
        foreach ($consumerParams["PARAMSHCM"] as $key => $value) {
            $this->hcmParameters->set($key, $value);
        }
    }
}

if (!isset($argv[1])) {
    die('Parametro de organizacao nao informado.');
}

$nrorg = $argv[1];
$pid = getmypid();

$paramsApolo = FPA::getParamsApolo();
        
$url = $paramsApolo['URL'];
$port = $paramsApolo['PORT'];
$user = $paramsApolo['USER'];
$password = $paramsApolo['PASSWORD'];
$broke = $paramsApolo['BROKE'];

$connection = new \Zeedhi\Queue\Connection($url, $port, $user, $password, $broke);

$consumer = new MigracaoXmlConsumer($connection);
$consumer->setQueueName(MigracaoXmlConsumer::QUEUE_NAME_MIGRACAO.$nrorg, $pid);
$consumer->run();