<?php
namespace HCM\FPA\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\Repositories;
use HCM\Geral\Services\VinculoServices;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\DateUtil;
use HCM\FPA\Services\FechamentoMensalService;
use HCM\Geral\Services\EstruturasService;

class FechamentoMensalController extends Crud {

    const Alfabetica    = 'ALFABETICA';
    const Data_Admissao = 'DATA_ADMISSAO';
    const Matricula     = 'MATRICULA';
    const Estrutura     = 'ESTRUTURA';
    protected $dataSourceGetFechamentoMensal = 'fpaFormCalculo_getFechamentoMensal';
    protected $dataSourceSetFechamentoMensal = 'fpaFormCalculo_setFechamentoMensal';
    protected $parameterBagColumns = array('NRORG', 'CDOPERADOR', 'IDTPFORMCALCULO');
    
    public function __construct (Manager $dataSourceManager, 
        VinculoServices $vinculoServices, HCMParameters $HCMParameters, FechamentoMensalService $fechamentoMensalService, EstruturasService $estruturasService, EntityManager $entityManager){
        $this->vinculoServices          = $vinculoServices;
        $this->hcmParameters            = $HCMParameters;
        $this->fechamentoMensalService  = $fechamentoMensalService;
        $this->estruturasServices       = $estruturasService;
        $this->entityManager            = $entityManager;
        parent::__construct($dataSourceManager);
    }


    public function getFechamentoMensal (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetFechamentoMensal);
            $newFilterData = $this->fechamentoMensalService->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $rows = $dataSet->getRows();
            foreach($rows as $key => &$row){
                if($row['DSORDEM'] == 'LISTA_VINCULO'){
                    $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                    foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                        $row['NRVINCULOM_LIST'][$chave] = intval($value);
                    
                    $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
                }    
                
                if($row['DSORDEM'] == 'LISTA_BANCOS'){
                    $row['LIST_CDBANCO'] = explode(",", $row['DSINICIO']);
                    foreach($row['LIST_CDBANCO'] as $chave => $value)
                        $row['LIST_CDBANCO'][$chave] = intval($value);
                    
                    $row['NMBANCO'] = explode(",", $row['NMBANCO']);
                }    
                

                if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                    $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                    foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                        $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                    
                    if($row['DSFIM']){
                        $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                        foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                            $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                        
                        $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                    }
                }   
            }
            
            $response->addDataSet(new DataSet($this->dataSourceSetFechamentoMensal, $rows));
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setFechamentoMensal (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetFechamentoMensal;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRORG'] = $row['NRORGTRAB'];
                    $row['NRFORMCALCULO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMCALCULO', 1, 12, true);
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }

                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }

                if(!array_key_exists('CDOPERADOR', $row)){
                    $row['CDOPERADOR'] = $row['CDOPERADOR_LOG'];
                }
 
                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
                
                if(!array_key_exists('IDDIFSAL', $row)){
                    $row['IDDIFSAL'] = null;
                }
                
                if(!array_key_exists('IDDIFFERIAS', $row)){
                    $row['IDDIFFERIAS'] = null;
                }
                
                if($row['DSORDEM'] === "LISTA_VINCULO" || $row['DSORDEM'] === "LISTA_BANCOS"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteFechamentoMensal (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetFechamentoMensal, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);

            $response->addDataSet(new DataSet($this->dataSourceSetFechamentoMensal, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function enviarRelatorio (DTO\Request\Row $row, DTO\Response $response){
        try{
            set_time_limit(0);
            $dataSet       = $row->getRow();
            $tipoRelatorio = $row->getRow()['TIPORELATORIO'];
            $directoryTemp = $this->hcmParameters->getTempDirectory().'/';
            
            $row->getRow()['OCORRECAL']      = $this->hcmParameters->getOcorrenciaCalculo();
            $row->getRow()['NRTPMODALIDCAL'] = $this->hcmParameters->getNrModalidadeCalculo();
    
            if (!isset($row->getRow()['DSOBSERVACOES']))
                $row->getRow()['DSOBSERVACOES'] = '';
            else 
                $row->getRow()['DSOBSERVACOES'] = urlencode(htmlentities($row->getRow()['DSOBSERVACOES']));
    
            $vinculos = $this->vinculoServices->retornaOnlyObjectVinculosCompetencia($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getDtCompetencia(), false, $dataSet['DSINICIO'], $dataSet['DSFIM'], $dataSet['DSORDEM'], true);
            // $vinculos = $this->vinculoServices->retornaOnlyObjectVinculosCompetencia($nrOrg, $mesCompetencia, false, $dsInicio, $dsFim, $ordem, true);
            
            if($row->getRow()['TIPORELATORIO'] == 'holerite') $this->enviarHoleriteViaEmail($row, $response, $vinculos, $directoryTemp);
            elseif($row->getRow()['TIPORELATORIO'] == 'reciboFerias') $this->enviarReciboFeriasViaEmail($row, $response, $vinculos, $directoryTemp);
            elseif($row->getRow()['TIPORELATORIO'] == 'informeRendimentos') $this->enviarInformeRendimentosViaEmail($row, $response, $vinculos, $directoryTemp);
            elseif($row->getRow()['TIPORELATORIO'] == 'pontoDemonstrativo') $this->enviarPontoDemonstrativoViaEmail($row, $response, $vinculos, $directoryTemp);
            
            $response->addNotification(new \Zeedhi\Framework\DTO\Response\Notification('Success', \Zeedhi\Framework\DTO\Response\Notification::TYPE_SUCCESS));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    private function enviarHoleriteViaEmail(&$row, &$response, $vinculos, $directoryTemp){
        $dataSet        = $row->getRow();
        $nrOrg          = $this->hcmParameters->getNrorgTrabalho();
        $mesCompetencia = $this->hcmParameters->getDtCompetencia();

        $returnVinculo = array();
        $returnDsEmailOper = null;
        $returnAmbos       = null;
        $returnSemLiquido  = null;
        $errorOnZip  = null;
        $validaTabelaOper = null;
        $sucesso = null;
        $message = null;
        $semCalculo = "";
        $semEmail = "";
        $groupEstruturasG = [];
        // Abaixo, é validado se é para enviar os holerites para os responsáveis informais. 
        // Caso seja, é agrupado os holerites por responsável. Caso não seja. Os holerites são enviados 
        // para seus respectivos vínculos.
        switch($dataSet['MENSAGEM']['RIEG']){
            case 'N':
                foreach ($vinculos as $vinculo) {
                    $fileName = $this->fechamentoMensalService->criarHolerite($directoryTemp, $vinculo, $mesCompetencia, $nrOrg, $returnSemLiquido, $row, $this->hcmParameters);
                    if($fileName != 'sem_calculo'){
                        if (file_exists($directoryTemp.$fileName)) {
                            $this->fechamentoMensalService->enviarHoleriteParaVinculo($directoryTemp, $row, $nrOrg, $mesCompetencia, $vinculo, $fileName, $returnVinculo, $returnDsEmailOper, $returnAmbos, $returnSemLiquido, $validaTabelaOper);
                        }else {
                            throw new \Exception(Messages::evaluateMessage(Messages::HCM01000_FPA_ERROR_SAVE_REPORT_VINCULO, $vinculo->getNrvinculom()), 1);                
                        }                    
                    }    
                }
                break;
            case 'Y':
                foreach ($vinculos as $vinculo) {
                    $fileName = $this->fechamentoMensalService->criarHolerite($directoryTemp, $vinculo, $mesCompetencia, $nrOrg, $returnSemLiquido, $row, $this->hcmParameters);
                    if($fileName != 'sem_calculo'){
                        if (file_exists($directoryTemp.$fileName)) {
                            $nrestrutgeren = $vinculo->historico->getNrestrutgeren();
                            $dados = $this->estruturasServices->retornaDadosResponsavelInformal($nrOrg, $vinculo->historico->getNrestrutgeren(), $mesCompetencia);
                            if($dados){
                                if(isset($groupEstruturasG["$nrestrutgeren"])){
                                    $groupEstruturasG["$nrestrutgeren"]['holerites'] .= $fileName.',';
                                }else{
                                    $groupEstruturasG["$nrestrutgeren"] = ['email'=>$dados['CDCOMUNICAPARC'], 'nome'=>$dados['NMPESSOA'], 'holerites'=>$fileName.',', 'estruturaL'=>$vinculo->historico->getNrestrutlegal()];
                                }
                            }else
                                $semEmail .= $vinculo->historico->getNrestrutgeren().', ';
                        }
                    }else 
                        $semCalculo .= $vinculo->getNmvinculom().', ';
                }
                $semEmail = substr($semEmail, 0, -2);
                $semCalculo = substr($semCalculo, 0, -2);
                if(count($groupEstruturasG)>0){
                    $errorOnZip = $this->fechamentoMensalService->enviarHoleritesParaResponsavelInformal($directoryTemp, $row, $nrOrg, $mesCompetencia, $groupEstruturasG, $sucesso);
                    $this->fechamentoMensalService->excluirArquivos($directoryTemp, $groupEstruturasG);
                }else if($semCalculo && !$semEmail)
                    $message = Messages::evaluateMessage(Messages::HCM01000_FPA_NENHUM_VINCULO_COM_CALCULO, $semCalculo);
                else if(!$semCalculo && $semEmail){
                    $message = Messages::evaluateMessage(Messages::HCM01000_FPA_NENHUM_RESP_INFORM_COM_EMAIL, $semEmail);
                }
                else if($semCalculo && $semEmail)
                    $message = Messages::evaluateMessage(Messages::HCM01000_FPA_SEM_CALC_SEM_EMAIL, [$semCalculo, $semEmail]);

                break;
        }
        
        if(!empty($returnSemLiquido))
            $returnSemLiquido = substr($returnSemLiquido, 0, -2);

        if(!is_null($returnDsEmailOper)){
            if(!is_null($message)){
                $message .= '<br><br>';
            }
            $message .= Messages::evaluateMessage(Messages::HCM01000_FPA_VINC_EMAIL_NOTFOUND, $returnDsEmailOper);
        }
        
        if(!is_null($sucesso)){
            if(!is_null($message)){
                $message .= '<br><br>';
            }
            $message .= Messages::evaluateMessage(Messages::HCM01000_FPA_SEND_EMAIL_SUCCES, $sucesso);
        }

        if($errorOnZip){
            if(!is_null($message)){
                $message .= '<br><br>';
            }
            $message .= Messages::evaluateMessage(Messages::HCM01000_ERROR_ZIP, $errorOnZip);                
        }        

        if(!is_null($returnAmbos)){
            if(!is_null($message)){
                $message .= '<br><br>';
            }
            $message .= Messages::evaluateMessage(Messages::HCM01000_FPA_VINC_OPERADOR_NOTFOUND. ' ou forma de comunicação', $returnAmbos);
        }

        if(!is_null($returnSemLiquido)){
            if(!is_null($message)){
                $message .= '<br><br>';
            }
            $message .= Messages::evaluateMessage(Messages::HCM01000_FPA_VINC_SEM_LIQUIDO, $returnSemLiquido);
        }
        
        if(!is_null($validaTabelaOper)){
            if(!is_null($message)){
                $message .= '<br><br>';
            }
            $message .= Messages::evaluateMessage($validaTabelaOper, '');
        }        

        if(!is_null($message)){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($message));
        }
    }
    
    private function enviarReciboFeriasViaEmail(&$row, &$response, $vinculos, $directoryTemp){
        $message = "";
        $vincsSemDefinitivo = "";
        $sucesso = false;

        if(!$vinculos){
            $message .= "Nenhum vínculo encontrado.";
            $message .= '<br><br>';
        }    
        
        foreach ($vinculos as $vinculo) {
            $temFeriasDefinitivo = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->_temFeriasDefinitivoComp($vinculo->getNrvinculom(), $this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getDtCompetencia(), $this->hcmParameters->getDtCompAnterior());
            if(is_array($temFeriasDefinitivo) && isset($temFeriasDefinitivo['TEMDEFINITIVO']) && !empty($temFeriasDefinitivo['TEMDEFINITIVO'])){
                $filtro = array(
                    'P_NRORG' => $this->hcmParameters->getNrorgTrabalho(),
                    'P_DTMESCOMPETENC'=>$this->hcmParameters->getDtCompetencia(),
                    'P_NRVINCULO' => $vinculo->getNrvinculom(),
                    'P_DATAINICIO'=>$this->hcmParameters->getDtCompAnterior(),
                    'P_DATAFIM'=>$this->hcmParameters->getUltimoDiaMes(),
                );
                $urlReport = $this->fechamentoMensalService->getUrlRelatorio($vinculo->getNrvinculom(), 'recibo_ferias', $filtro);
                $fileNamePath = $this->fechamentoMensalService->gerarRelatorioGenerico($urlReport, 'recibo_ferias', $vinculo->getNrvinculom(), 'pdf');
                if (file_exists($fileNamePath)) {
                    $this->fechamentoMensalService->enviarRelatorioGenericoViaEmail($row, $this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getDtCompetencia(), $vinculo, $fileNamePath);
                    unlink($fileNamePath);
                    $sucesso = true;
                }else {
                    throw new \Exception(Messages::evaluateMessage(Messages::HCM01000_FPA_ERROR_SAVE_REPORT_VINCULO, $vinculo->getNrvinculom()), 1);                
                }
            }else{
                if(strlen($vincsSemDefinitivo) > 0) $vincsSemDefinitivo .= ", ";
                $vincsSemDefinitivo .= $vinculo->getNrvinculom();
            }
        }
    
        if($sucesso){
            $message .= "Envio Realizado com sucesso";
            $message .= '<br><br>';
        }
        
        if(strlen($vincsSemDefinitivo) > 0){
            $message .= "O(s) seguinte(s) vínculos(s) não possuem Férias Definitivas na Competência: ".$vincsSemDefinitivo;
            $message .= '<br><br>';
        }
        
        if(!is_null($message)){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($message));
        }

    }
    
    private function enviarInformeRendimentosViaEmail(&$row, &$response, $vinculos, $directoryTemp){
        $message = "";
        $sucesso = false;
        $vincsSemDIRF = '';
        $nrMesCompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true)->format('m');

        if(!$vinculos){
            $message .= "Nenhum vínculo encontrado.";
            $message .= '<br><br>';
        }    

        if(intval($nrMesCompetenc) == 12){
            foreach ($vinculos as $vinculo) {
                $temLinhaMagneticoDirf = $this->entityManager->getRepository(Repositories::FPA_DIRFMAGNET)->_temMagneticoDirf($vinculo->getNrvinculom(), $this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getDtCompetencia());
                if(is_array($temLinhaMagneticoDirf) && isset($temLinhaMagneticoDirf['TEMLINHA']) && !empty($temLinhaMagneticoDirf['TEMLINHA'])){
                    $filtro = array(
                        'P_NRORG' => $this->hcmParameters->getNrorgTrabalho(),
                        'P_COMPETENCIA'=>$this->hcmParameters->getDtCompetencia(),
                        'P_VINCULO' => $vinculo->getNrvinculom(),
                    );
                    $urlReport = $this->fechamentoMensalService->getUrlRelatorio($vinculo->getNrvinculom(), 'informe_rendimentos', $filtro);
                    $fileNamePath = $this->fechamentoMensalService->gerarRelatorioGenerico($urlReport, 'informe_rendimentos', $vinculo->getNrvinculom(), 'pdf');
                    if (file_exists($fileNamePath)) {
                        $this->fechamentoMensalService->enviarRelatorioGenericoViaEmail($row, $this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getDtCompetencia(), $vinculo, $fileNamePath);
                        unlink($fileNamePath);
                        $sucesso = true;
                    }else {
                        throw new \Exception(Messages::evaluateMessage(Messages::HCM01000_FPA_ERROR_SAVE_REPORT_VINCULO, $vinculo->getNrvinculom()), 1);                
                    }                    
                }else{
                    if(strlen($vincsSemDIRF) > 0) $vincsSemDIRF .= ", ";
                    $vincsSemDIRF .= $vinculo->getNrvinculom();
                }
            }
    
            if($sucesso){
                $message .= "Envio Realizado com sucesso";
                $message .= '<br><br>';
            }
        }else{
            $message .= "O mês da competência deve ser 12.";
            $message .= '<br><br>';
        }
        
        if(strlen($vincsSemDIRF) > 0){
            $message .= "O(s) seguinte(s) vínculos(s) não possuem linha(s) no magnético da DIRF: ".$vincsSemDIRF;
            $message .= '<br><br>';
        }

        if(!is_null($message)){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($message));
        }

    }
    
    private function enviarPontoDemonstrativoViaEmail(&$row, &$response, $vinculos, $directoryTemp)
    {
        $message = "";
        $sucesso = false;

        
        if(!$vinculos){
            $message .= "Nenhum vínculo encontrado.";
            $message .= '<br><br>';
        }
        
        $dtultdiacompetencia = DateUtil::getUltimoDiaDoMes($this->hcmParameters->getDtCompetencia())->format(DateUtil::FORMATO_BRASILEIRO);
        
        // Periodo de Apuração CLT
        $ptoPeriodoApuracaoClt = current($this->entityManager->getRepository(Repositories::PTO_PERIODOAPURACAO)->getPeriodoApuracao(
            $this->hcmParameters->getNrorgTrabalho(), $dtultdiacompetencia, 'FECHADO', null, 1
        ));
        // Periodo de Apuração Estagiario
        $ptoPeriodoApuracaoEst = current($this->entityManager->getRepository(Repositories::PTO_PERIODOAPURACAO)->getPeriodoApuracao(
            $this->hcmParameters->getNrorgTrabalho(), $dtultdiacompetencia, 'FECHADO', null, 2
        ));
        // Periodo de Apuração Socio
        $ptoPeriodoApuracaoSoc = current($this->entityManager->getRepository(Repositories::PTO_PERIODOAPURACAO)->getPeriodoApuracao(
            $this->hcmParameters->getNrorgTrabalho(), $dtultdiacompetencia, 'FECHADO', null, 3
        ));
        // Periodo de Apuração Autonomo
        $ptoPeriodoApuracaoAut = current($this->entityManager->getRepository(Repositories::PTO_PERIODOAPURACAO)->getPeriodoApuracao(
            $this->hcmParameters->getNrorgTrabalho(), $dtultdiacompetencia, 'FECHADO', null, 10
        ));

        foreach ($vinculos as $vinculo) {
            switch ($vinculo->getNrtipovinculom()) {
                case 1:
                    $ptoPeriodoApuracao = $ptoPeriodoApuracaoClt;
                    break;
                case 2:
                    $ptoPeriodoApuracao = $ptoPeriodoApuracaoEst;
                    break;
                case 3:
                    $ptoPeriodoApuracao = $ptoPeriodoApuracaoSoc;
                    break;
                case 10:
                    $ptoPeriodoApuracao = $ptoPeriodoApuracaoAut;
                    break;
                default:
                    $ptoPeriodoApuracao = false;
            }
            
            // Se não existir periodo de apuração para o tipo de vínculo não envio o demonstrativo
            if (!$ptoPeriodoApuracao || !$this->fechamentoMensalService->possuiPonto($this->hcmParameters->getNrorgTrabalho(),$vinculo->getNrvinculom(),$ptoPeriodoApuracao->getNrperiodoapuracao())) {
                continue;
            }
            
            $filtro = array(
                'P_NRORG' => $this->hcmParameters->getNrorgTrabalho(),
                'P_NRORG_PADRAO' => 0,
                'P_COMPETENCIA'=> $this->hcmParameters->getDtCompetencia(),
                'P_VINCULO' => $vinculo->getNrvinculom(),
                'P_TPVINCULO' => $vinculo->getNrtipovinculom(),
                'P_ESTRUTLEGAL' => null,
                'P_ESTRUTGEREN' => null,
                'P_PERIODO' => $ptoPeriodoApuracao->getNrperiodoapuracao(),
                'P_NROCORRECAL' => 1,
                'P_OCORRECAL' => 1,
                'P_MODALIDADECAL' => 1,
                'P_NRTPMODALIDCAL' => 1,
                'CDOPERADOR' => $this->hcmParameters->getCdoperador(),
                'NMOPERADOR' => $this->hcmParameters->getNmoperador(),
            );
            
            $urlReport = $this->fechamentoMensalService->getUrlRelatorio($vinculo->getNrvinculom(), 'ponto_demonstrativo', $filtro);
            $fileNamePath = $this->fechamentoMensalService->gerarRelatorioGenerico($urlReport, 'ponto_demonstrativo', $vinculo->getNrvinculom(), 'pdf');
            if (file_exists($fileNamePath)) {
                $this->fechamentoMensalService->enviarRelatorioGenericoViaEmail($row, $this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getDtCompetencia(), $vinculo, $fileNamePath);
                unlink($fileNamePath);
                $sucesso = true;
            }else {
                throw new \Exception(Messages::evaluateMessage(Messages::HCM01000_FPA_ERROR_SAVE_REPORT_VINCULO, $vinculo->getNrvinculom()), 1);
            }
        }

        if($sucesso){
            $message .= "Envio Realizado com sucesso";
            $message .= '<br><br>';
        } else {
            $message .= "Nenhum demonstrativo foi enviado";
            $message .= '<br><br>';
        }

        if(!is_null($message)){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($message));
        }

    }
}