<?php

namespace HCM\FPA\Registro;

class RegistroContratoHomolognet extends RegistroHomolognet{

    const CONTRATOS = 'Contratos';
    const CONTRATO = 'Contrato';

    // Estes dados constam no layout do empregado
    const CBO = 'NRCbo';
    const CNPJ_CEES = 'NRCnpjEslCees';
    const COD_SINDICAL_CEES = 'NRCodigoSindicalEslCees';
    const DATA_BASE_CAT_PROF = 'DTBaseCategoriaProfissional';

    const RURAL = '2';
    const URBANO = '1';

    const SIM = 'S';
    const NAO = 'N';

    const DATA_ADMISSAO = 'DTAdmissao';
    const JORNADA_SEMANAL = 'NRTrabalhoSemanal';
    const DATA_AVISO_PREVIO = 'DTAvisoPrevio';
    const DATA_AFASTAMENTO = 'DTAfastamento';
    const CAUSA_AFASTAMENTO = 'TPCausaAfastamento';
    const TIPO_CONTRATO_TRAB = 'TPContratoTrabalho';
    const COMPROVOU_EMP_AVISO_PREVIO = 'TPComprovacaoEmpregoDuranteAvisoPrevio';
    const DISPENSADO_AVISO_PREVIO = 'TPEmpregadoDispensadoAvisoPrevio';
    const VALOR_PENSAO_ALIMENTICIA = 'VLPensaoAlimenticiaTRCT';
    const PERCENTUAL_PENSAO_ALIMENTICIA = 'PCPensaoAlimenticiaTRCT';
    const FGTS_PENSAO_ALIMENTICIA = 'PCPensaoAlimenticiaFGTS';
    const CATEGORIA_TRABALHADOR = 'TPCategoriaTrabalhador';
    const DEPENDENTES_IR = 'NRDependentesIR';
    const DEPENDENTES_SF = 'NRDependentesSalarioFamilia';
    const DATA_TERMINO_PRAZO_DETERMINADO = 'DTTerminoContratoPrazoDeterminado';
    const TIPO_CONTRATO_TEMPO_PARCIAL = 'TPContratoTempoParcial';
    const TIPO_TRABALHADOR = 'TPTrabalhador';
    const DATA_QUITACAO_RESCISAO = 'DTQuitacaoRescisao';

    const FERIAS = 'Ferias';
    const DECIMOS_TERCEIROS = 'DecimosTerceiros';
    const MOVIMENTACOES = 'Movimentacoes';
    const FINANCEIRO = 'Financeiro';
    const DESCONTO = 'Desconto';

    public $tpContratoTrabalho;

    private $_vinculo;
//    private $_ferias;
//    private $_decimosTerceiros;
//    private $_movimentacoes;
//    private $_financeiro;
//    private $_desconto;


    public function geraRegistroContrato($vinculo, $pensaojudicial){
        $this->_vinculo = $vinculo;
        $this->_geraRegistro($pensaojudicial);
    }


    private function _geraRegistro($pensaojudicial){

        parent::geraRegistro(null, null, null);

        // Campo 31 - Tamanho 06 - Codigo CBO - Dado consta nos dados do Empregado conforme layout
        $this->_atribuiCbo();

        // Campo 32 - Tamanho 14 - CNPJ Entidade Sindical - Dado consta nos dados do Empregado conforme layout
        $this->_atribuiCnpjEntidadeSindical();

        // Campo 33 - Tamanho 15 - Codigo Sindical de Entidade Sindical - Dado consta nos dados do Empregado conforme layout
        $this->_atribuiCodigoEntidadeSindical();

        // Campo 34 - Tamanho 04 - Data base da categoria profissional - Dado consta nos dados do Empregado conforme layout
        $this->_atribuiDataBaseCategoriaProfissional();

        // Campo 35 - Tamanho 08 - Data Admissao - Inicio de vigencia do contrato
        $this->_atribuiDataAdmissao();

        // Campo 36 - Tamanho 02 - Jornada de trabalho semanal em horas
        $this->_atribuiJornadaSemanal();

        //Campo 37 - Tamanho 08 - Data do aviso previo
        $this->_atribuiDataAvisoPrevio();

        // Campo 38.1 - Tamanho 08 - Data do ultimo dia do contrato de trabalho
        $this->_atribuiDataAfastamento();

        // Campo 38.2 - Tamanho 08 - Data Prevista de Quitação da Rescisão - NÃO OBRIGATÓRIO
        $this->_atribuiDataQuitacaoRescisao();

        // Campo 39 - Tamanho 03 - Tipo da causa do afastamento
        $this->_atribuiCausaAfastamento();

        // Campo 40 - Tamanho 01 - Tipo de contrato de trabalho
        $this->_atribuiContratoTrabalho();

        // Campo 41 - Tamanho 01 - Empregado comprovou ter novo emprego durante o período do Aviso Prévio?
        $this->_atribuiComprovacaoEmpregoAvisoPrevio();

        // Campo 42 - Tamanho 01 - Empregador dispensou o empregado do aviso prévio?
        $this->_atribuiDispensaAvisoPrevio();

//        Campo 43.1 - Tamanho 04 - Pensão Alimentícia % - TRCT - NÃO OBRIGATÓRIO
        $this->_atribuiPercentualPensaoAlimenticia();

//        Campo 43.11 - Tamanho 14 - Pensão Alimentícia (em R$) - TRCT
        $this->_atribuiValorPensaoAlimenticia($pensaojudicial);

//          Campo 43.2 - Tamanho 04 - Pensão Alimentícia % - FGTS - NÃO OBRIGATÓRIO
        $this->_atribuiFGTSPensaoAlimenticia();

        // Campo 44 - Tamanho 02 - Codigo da categoria do trabalhador
        $this->_atribuiCategoriaTrabalhador();

        // Campo 45 - Tamanho 03 - Numero de dependentes de Imposto de Renda
        $this->_atribuiNumeroDependentesIR();

        // Campo 46 - Tamanho 03 - Numero de dependentes de Salario Familia
        $this->_atribuiNumeroDependentesSF();

//        // Campo 47 - Tamanho 08 - Data de término do contrato por prazo determinado - NÃO OBRIGATÓRIO
        $this->_atribuiDataTerminoPrazoDeterminado();

        // Campo 48 - Tamanho 01 - Informar se o contrato é a tempo parcial (jornada de trabalho)
        $this->_atribuiTipoContratoTempoParcial();

        // Campo 49 - Tamanho 01 - Especificar se o trabalhador é urbano ou rural.
        $this->_atribuiTipoTrabalhador();


//        $this->_atribuiFerias();
//
//        $this->_atribuiDecimosTerceiros();
//
//        $this->_atribuiMovimentacoes();
//
//        $this->_atribuiFinanceiro();
//
//        $this->_atribuiDesconto();


    }


//    public function setDecimosTerceiros($decimosTerceiros) {
//        $this->_decimosTerceiros = $decimosTerceiros;
//    }
//
//
//    public function setDesconto($desconto) {
//        $this->_desconto = $desconto;
//    }
//
//
//    public function setFerias($ferias) {
//        $this->_ferias = $ferias;
//    }
//
//
//    public function setFinanceiro($financeiro) {
//        $this->_financeiro = $financeiro;
//    }
//
//
//    public function setMovimentacoes($movimentacoes) {
//        $this->_movimentacoes = $movimentacoes;
//    }


    private function _atribuiCbo(){
        $dado = $this->_vinculo->cbo;
        if (empty($dado) === false){
            $tag = self::CBO.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 6);
        }
    }

    private function _atribuiCnpjEntidadeSindical(){
        $dado = $this->_vinculo->cnpjEntidadeSindical;
        if (empty($dado) === false){
            $tag = self::CNPJ_CEES.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 14);
        }
    }

    private function _atribuiCodigoEntidadeSindical(){
        // Atribuir com 3 zeros a esquerda.
        $dado = '000'.$this->_vinculo->codEntidadeSindical;
        if (empty($dado) === false){
            $tag = self::COD_SINDICAL_CEES.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 15);
        }
    }

    private function _atribuiDataBaseCategoriaProfissional(){
        $dado = $this->_vinculo->dtBaseSindical;
        //@todo verificar a definição da data base na estrutura
        if (empty($dado) === true) {
            $dado = '0101';
        }
        $tag = self::DATA_BASE_CAT_PROF.parent::CARACTER_ATRIBUI;
        $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 4);
    }

    private function _atribuiDataAdmissao(){
        // Informe de acordo com o seguinte formato: aaaammdd
        if ($this->_vinculo->getDtadmissaovinc()){
            $dataAdmissao = new TkDateTime($this->_vinculo->getDtadmissaovinc());
            $dado = $dataAdmissao->toString(TkDateTime::YEAR.TkDateTime::MONTH.TkDateTime::DAY);
            if (empty($dado) === false){
                $tag = self::DATA_ADMISSAO.parent::CARACTER_ATRIBUI;
                $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 8);
            }
        }
    }

    private function _atribuiJornadaSemanal(){
        $dado = $this->_vinculo->jornadaSemanal;
        if (empty($dado) === false){
            $tag = self::JORNADA_SEMANAL.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 2);
        }
    }

    private function _atribuiDataAvisoPrevio(){

        if ($this->_vinculo->getDtavisoprevio()){
            $dataAvisoPrevio = new TkDateTime($this->_vinculo->getDtavisoprevio());
            $dado = $dataAvisoPrevio->toString(TkDateTime::YEAR.TkDateTime::MONTH.TkDateTime::DAY);
            if (empty($dado) === false){
                $tag = self::DATA_AVISO_PREVIO.parent::CARACTER_ATRIBUI;
                $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 8);
            }
        }
    }

    private function _atribuiDataAfastamento(){
        if ($this->_vinculo->getDtrescisaovinc()){
            $dataAfastamento= new TkDateTime($this->_vinculo->getDtrescisaovinc());
            $dado = $dataAfastamento->toString(TkDateTime::YEAR.TkDateTime::MONTH.TkDateTime::DAY);
            if (empty($dado) === false){
                $tag = self::DATA_AFASTAMENTO.parent::CARACTER_ATRIBUI;
                $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 8);
            }
        }
    }

    private function _atribuiCausaAfastamento(){
        // Verificar Ref.1 do layout
        $dado = $this->_vinculo->codAfastamento;
        if (empty($dado) === false){
            $tag = self::CAUSA_AFASTAMENTO.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 3);
        }
    }

    private function _atribuiContratoTrabalho(){

        // Informe de acordo com o seguinte domínio:
        // 1. Contrato de trabalho por prazo indeterminado.
        // 2. Contrato de trabalho por prazo determinado com cláusula assecuratória de direito recíproco de rescisão antecipada.
        // 3. Contrato de trabalho por prazo determinado sem cláusula assecuratória de direito recíproco de rescisão antecipada.

        $dado = $this->_vinculo->contratoTrabalho;
        $this->tpContratoTrabalho = $dado;
        if (empty($dado) === true) {
            $dado = '1';
        }
        $tag = self::TIPO_CONTRATO_TRAB.parent::CARACTER_ATRIBUI;
        $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 1);

    }

    private function _atribuiComprovacaoEmpregoAvisoPrevio(){
        /* · Informe de acordo com o seguinte domínio: 1. SIM / 2. NÃO
        * · Opções mutuamente excludentes.
        * · Este campo somente será aplicável a determinados tipos de rescisão, onde o código de afastamento for referente à
        *   “SJ2 - Despedida sem justa causa, pelo empregador” e “RA2 – “Rescisão Antecipada, pelo empregador, do contrato de
        *   trabalho por prazo determinado” com cláusula assecuratória de direito recíproco de rescisão antecipada”,
        *   de acordo com informação fornecida para o tipo de contrato no campo 40 (domínio = 2).
        * · Não poderá ser nulo para os casos em que este campo for obrigatório.
        */

        $dado = $this->_vinculo->idComprovouAviso;
        if (empty($dado) === false){
            if ($dado === self::SIM) {
                $dado = '1';
            } else {
                $dado = '2';
            }
            $tag = self::COMPROVOU_EMP_AVISO_PREVIO.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 1);

        }
    }

    private function _atribuiDispensaAvisoPrevio(){
        /* · Informe de acordo com o seguinte domínio: 1. SIM / 2. NÃO
        * · Opções mutuamente excludentes.
        * · Este campo somente será aplicável a determinados tipos de rescisão, onde o código de afastamento for referente à
        *   “SJ1 – Rescisão contratual a pedido do empregado” e “RA1 - Rescisão antecipada pelo empregado, do contrato de
        *   trabalho por prazo determinado” com cláusula assecuratória de direito recíproco de rescisão antecipada”,
        *   de acordo com informação fornecida para o tipo de contrato no campo 40 (domínio = 2).
        * · Não poderá ser nulo para os casos em que este campo for obrigatório.
        */
        $dado = $this->_vinculo->idDispensouAviso;
        if (empty($dado) === false){
            if ($dado === self::SIM) {
                $dado = '1';
            } else {
                $dado = '2';
            }
            $tag = self::DISPENSADO_AVISO_PREVIO.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 1);
        }
    }

    private function _atribuiValorPensaoAlimenticia($pensaojudicial){
        /* · Deve ser informado neste campo o percentual que será descontado na rescisão contratual para retenção da pensão alimentícia no TRCT.
        * · Preencher em percentual e com duas casas decimais.
        * · Se preenchido este campo, não preencher o campo seguinte (43.11 Pensão Alimentícia (em R$) – TRCT).
        */
        $dado = $pensaojudicial[RegistroHomolognet::HOMOLOGNET_PENSAO_JURIDICA];
        if (empty($dado) === true) {
            $dado = '0.00';
        }
        $tag = self::VALOR_PENSAO_ALIMENTICIA.parent::CARACTER_ATRIBUI;
        $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 4);

    }

    private function _atribuiPercentualPensaoAlimenticia(){
        /* · Informar este campo apenas se a pensão alimentícia foi estabelecida em “valor monetário fixo”, e não em percentual.
        * · Informar com duas casas decimais.
        * · Se preenchido este campo, não preencher o campo anterior (43.1 Pensão Alimentícia % – TRCT).
        */
        $dado = '';
        if (empty($dado) === true) {
            $dado = '0.00';
        }
        $tag = self::PERCENTUAL_PENSAO_ALIMENTICIA.parent::CARACTER_ATRIBUI;
        $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 14, true);

    }

    private function _atribuiFGTSPensaoAlimenticia(){
        /* · Deve ser informado neste campo o percentual que será descontado no FGTS pela Caixa Econômica Federal,
        *   para retenção da cota-parte do FGTS relativa à pensão alimentícia.
        * · Preencher em percentual e com duas casas decimais.
        */
        $dado = '';
        if (empty($dado) === true) {
            $dado = '0.00';
        }
        $tag = self::FGTS_PENSAO_ALIMENTICIA.parent::CARACTER_ATRIBUI;
        $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 4, true);

    }

    private function _atribuiCategoriaTrabalhador(){
        // Verificar tabela de categoria do trabalhador Ref. 2 no layout
        $dado = $this->_vinculo->codCategTrab;
        if (empty($dado) === false){
            $tag = self::CATEGORIA_TRABALHADOR.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 2);
        }
    }

    private function _atribuiNumeroDependentesIR(){
        $dado = $this->_vinculo->numeroDependentesIR;
        if (empty($dado) === true) {
            $dado = '000';
        }
        $tag = self::DEPENDENTES_IR.parent::CARACTER_ATRIBUI;
        $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 3);

    }

    private function _atribuiNumeroDependentesSF(){
        $dado = $this->_vinculo->numeroDependentesSF;
        if (empty($dado) === true) {
            $dado = '000';
        }
        $tag = self::DEPENDENTES_SF.parent::CARACTER_ATRIBUI;
        $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 3);

    }

    private function _atribuiDataTerminoPrazoDeterminado(){
        /* · Informe de acordo com o seguinte formato: aaaammdd.
        * · Este campo será obrigatório se o campo 40 tiver o domínio informado com valor igual a: 2 ou 3 (“Contrato de
        *   trabalho por prazo determinado com cláusula assecuratória de direito recíproco de rescisão antecipada” e “Contrato de
        *   trabalho por prazo determinado sem cláusula assecuratória de direito recíproco de rescisão antecipada”, respectivamente)
        */
        if ($this->_vinculo->getDtrescisaovinc()){
            $tipocontrato = array('2','3');
            if (array_search($this->_vinculo->getNrtpdemissao(), $tipocontrato) !== false) {
                $dataTerminoPrazo = new TkDateTime($this->_vinculo->getDtrescisaovinc());
                $dado = $dataTerminoPrazo->toString(TkDateTime::YEAR.TkDateTime::MONTH.TkDateTime::DAY);
                if (empty($dado) === false){
                    $tag = self::DATA_TERMINO_PRAZO_DETERMINADO.parent::CARACTER_ATRIBUI;
                    $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 8);
                }
            }
        }
    }

    private function _atribuiTipoContratoTempoParcial(){
        // · Informe de acordo com o seguinte domínio: 1. SIM / 2. NÃO
        $dado = $this->_vinculo->contratoParcial;
        if (empty($dado) === false){
            $tag = self::TIPO_CONTRATO_TEMPO_PARCIAL.parent::CARACTER_ATRIBUI;
            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 1);
        }
    }

    private function _atribuiTipoTrabalhador(){

        //informe de acordo com o seguinte domínio: 1. URBANO / 2. RURAL
        //3  - Trab. Rural - Pessoa Jurídica
        //4  - Trab. Rural - Pessoa Física
        //12 - Rural-Pes juridica/determinado
        //13 - Rural-Pes física/determinado

        $tipovinculo = array('3','4','12','13');
        if (array_search($this->_vinculo->getNrvinculoempreg(), $tipovinculo) !== false) {
            $dado = self::RURAL;
        }else{
            $dado = self::URBANO;
        }

        $tag = self::TIPO_TRABALHADOR.parent::CARACTER_ATRIBUI;
        $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 1);

    }

    private function _atribuiDataQuitacaoRescisao(){
        if ($this->_vinculo->getDtrescisaovinc()){
            $dataQuitacaoRescisao = new TkDateTime($this->_vinculo->getDtrescisaovinc());
            $dado = $dataQuitacaoRescisao->toString(TkDateTime::YEAR.TkDateTime::MONTH.TkDateTime::DAY);
            if (empty($dado) === false){
                $tag = self::DATA_QUITACAO_RESCISAO.parent::CARACTER_ATRIBUI;
                $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $dado, 8);
            }
        }
    }


//    private function _atribuiFerias(){
//        if (empty($this->_ferias) === false){
//            $tag = self::FERIAS.parent::CARACTER_ATRIBUI;
//            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $this->_ferias, strlen($this->_ferias));
//
//            $tag = parent::CARACTER_FINALIZADOR;
//            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, null, 0);
//        }
//    }
//
//    private function _atribuiDecimosTerceiros(){
//        if (empty($this->_decimosTerceiros) === false){
//            $tag = self::DECIMOS_TERCEIROS.parent::CARACTER_ATRIBUI;
//            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $this->_decimosTerceiros, strlen($this->_decimosTerceiros));
//
//            $tag = parent::CARACTER_FINALIZADOR;
//            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, null, 0);
//        }
//    }
//
//    private function _atribuiMovimentacoes(){
//        if (empty($this->_movimentacoes) === false){
//            $tag = self::MOVIMENTACOES.parent::CARACTER_ATRIBUI;
//            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $this->_movimentacoes, strlen($this->_movimentacoes));
//
//            $tag = parent::CARACTER_FINALIZADOR;
//$this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, null, 0);
//}
//}
//
//    private function _atribuiFinanceiro(){
//        if (empty($this->_financeiro) === false){
//            $tag = self::FINANCEIRO.parent::CARACTER_ATRIBUI;
//            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $this->_financeiro, strlen($this->_financeiro));
//
//            $tag = parent::CARACTER_FINALIZADOR;
//            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, null, 0);
//        }
//    }
//
//    private function _atribuiDesconto(){
//        if (empty($this->_desconto) === false){
//            $tag = self::DESCONTO.parent::CARACTER_ATRIBUI;
//            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, $this->_desconto, strlen($this->_desconto));
//
//            $tag = parent::CARACTER_FINALIZADOR;
//            $this->atribuiDadoXML($tag, parent::CARACTER_SEPARADOR, null, 0);
//        }
//    }
}
