<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\FPA\Services\ValorReferenciaService;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Repository\AlteSalarioRepository;
use Folha\Entities\GpeVinculom;
use HCM\FPA\Services\ExportacaoExcelService;
use HCM\Util\FPA;
use Zeedhi\Framework\HTTP\Kernel;

class AlteSalarioService {

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, ValorReferenciaService $valorReferenciaService, VinculoServices $vinculoServices, AlteSalarioRepository $alteSalarioRepository, ExportacaoExcelService $exportacaoExcelService, Kernel $httpKernel) {
        $this->entityManager          = $entityManager;
        $this->hcmParameters          = $hcmParameters;
        $this->valorReferenciaService = $valorReferenciaService;
        $this->vinculoServices        = $vinculoServices;
        $this->alteSalarioRepository  = $alteSalarioRepository;
        
        $this->gpe_Altesalario = Repositories::GPE_ALTESALARIO;
        $this->exportacaoExcelService = $exportacaoExcelService;
        $this->httpKernel = $httpKernel;
    }
    
    public function baixarPlanilhaSalarial($nrorg, $nmarquivo) {
        $cabecalho = array(
            "Nr. vinculo",
            "Matrícula",
            "Matrícula eSocial",
            "Nome",
            "Dt. Alteração",
            "Numero Ocorrencia",
            "Tipo Alteração",
            "Salário",
            "Tipo Salário",
            "Nr Nivel Faixa",
            "Observação",
            "Dt Base"
        );
        $dadosExemplo[0] = array(
            "353",
            "",
            "",
            "Nome do Vinculo",
            "01/01/2021",
            "1",
            "1",
            "1000",
            "MENSAL",
            "1",
            " ",
            " "
        );
        
        return $this->exportacaoExcelService->gravarTemplate_1($nrorg, $cabecalho, $nmarquivo, $dadosExemplo);
    }
    
    public function importarPlanilhaSalarial($row) {
        //Dados básicos
        $nrorg       = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $messages    = array(); //Mensages de erro, se estiver vazio não houve erros
        
        //Aquisição dos dados de exportação
        $dados = $this->getDadosPlanilhaSalarial($row['DSPATH'][0]);
        //var_dump($dados);die;
        foreach($dados as $lin => $dado) {
            $res = $this->inserePessoaPlanilhaSalarial($lin + 2, $nrorg, $nrorgPadrao,
                $dado['NRVINCULOM'], $dado['CDMATRICULA'], $dado['CDMATRICULAESOCIAl'], $dado['NMPESSOA'], $dado['DTALTERACAO'],
                $dado['NMOCORRENCIA'], $dado['TPALTERACAO'], $dado['SALARIO'],
                $dado['TPSALARIO'], $dado['NRNIVELFAIXA'], $dado['OBS'], $dado['DTBASEDISSIDIO']);
            if(count($res)>0)
                foreach($res as $v)
                    $messages[] = $v; 
        }
        return $messages;
    }
     public function getDadosPlanilhaSalarial($path) {
        //Os ID's (campos) atribuídos aos dados da planilha
        $head = array(
            'NRVINCULOM',
            'CDMATRICULA',
            'CDMATRICULAESOCIAl',
            'NMPESSOA',
            'DTALTERACAO',
            'NMOCORRENCIA',
            'TPALTERACAO',
            'SALARIO',
            'TPSALARIO',
            'NRNIVELFAIXA',
            'OBS',
            'DTBASEDISSIDIO'
            );
        //Colunas de data
        $dateColumns = array('DTALTERACAO','DTBASEDISSIDIO');
        
        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);

        return $this->exportacaoExcelService->preparaInfo_generic($file, $head, array('NRVINCULOM','CDMATRICULA','CDMATRICULAESOCIAl' ), $dateColumns);
    }
    
    public function inserePessoaPlanilhaSalarial($nrRow, $nrorg, $nrorgPadrao, $nrvinculom, $cdmatricula, $cdmatriculaesocial, $nmpessoa, $dtalteracao, $nrocorrencia, $tpalteracao, $salario, $tpsalario, $nrnivelfaixa, $obs, $dtbasedissidio) {
        $return = array();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $salario = floatval($salario);
        if($salario == 0)
           $return[] = array("msg" => "O Valor do salário não pode ser igual a $salario na linha $nrRow.");
        if($dtalteracao)
            $dtalteracao = DateUtil::getDataDeString($dtalteracao, DateUtil::FORMATO_BRASILEIRO, true);
        else{
            $return[] = array("msg" => "A data de alteração salarial não pode ser vazia, linha $nrRow.");
            return $return;
        }
        if($dtbasedissidio){
            $dtbasedissidio = DateUtil::getDataDeString($dtbasedissidio, DateUtil::FORMATO_BRASILEIRO, true);
        }
        try {
            //var_dump($dtbasedissidio,$dtalteracao);die;
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(
                array('nrorg'=>$nrorg, 'nrvinculom'=>$nrvinculom));
            if($vinculom == null)
                if($cdmatricula){
                    $objVinculom=GpeVinculom::findOneByNrorgAndCdmatricula($nrorg, $cdmatricula);
                    if ($objVinculom == null)
                         $return[] = array("msg"=>"O código da matrícula do vínculo não foi encontrado na linha $nrRow.");
                    else
                        $nrvinculom = $objVinculom->getNrvinculom();
                }else{
                    if($cdmatriculaesocial){
                        $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaVinculoPorMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true), null);
                        if ($objVinculom == null)
                             $return[] = array("msg"=>"O código da matrícula do ESocial não foi encontrado na linha $nrRow.");
                        else
                            $nrvinculom = $objVinculom->getNrvinculom();
                    }else{
                        $return[] = array("msg"=>"O código do vínculo, da matrícula do vínculo ou da matrícula do ESocial não foram encontrados. É necessário o preenchimento de pelo menos um desses campos.");
                    }
                }
                
            $validaNrTpSalario = $this->entityManager->getRepository(Repositories::GPE_TIPOSALARIO)->findBy(
                    array('nrorg'=>array($nrorgPadrao,$nrorg),
                    'cdtiposalario'=>$tpsalario
                ));
            if (count($validaNrTpSalario)==0) {
                $return[] = array("msg" => "Tipo de salário $tpsalario inválido na linha $nrRow.");
            }
            $nrtipoaltera = $this->entityManager->getRepository(Repositories::GPE_TIPOALTERA)->findOneBy(
                array('nrorg'=>array($nrorgPadrao,$nrorg),
                'nrtipoaltera'=>$tpalteracao
                ));
            if (is_null($nrtipoaltera)) {
                $return[] = array("msg" => "Tipo de alteração ($tpalteracao) inválida na linha $nrRow.");
            }
            if($dtalteracao == null){
                $return[] = array("msg" => "Data de alteração inválida na linha $nrRow.");
                return $return;
            }
            if($dtbasedissidio > $dtalteracao){
                $return[] = array("msg" => "A Dt Base não pode ser superior a data de alteração. Erro na linha $nrRow.");
                return $return;
            }
            $verificadtalteracao = $this->entityManager->getRepository(Repositories::GPE_ALTESALARIO)->findBy(
                    array('nrorg' => $nrorg,
                    'nrvinculom' => $nrvinculom,
                    'nrocorrencia' => $nrocorrencia,
                    'dtaltesalario' => $dtalteracao
                ));
            if (!empty($verificadtalteracao)) {
                $dtalteracao = substr($dtalteracao, 0, 10);
                $return[] = array("msg" => "Vínculo já possui alteração salarial na data $dtalteracao, na linha $nrRow.");
            }
            if(count($return)> 0)
                return $return;
            $insereplanilha = $this->nova($nrorg, $nrvinculom, $dtalteracao, $nrocorrencia , $tpalteracao, $salario, $tpsalario, '1', $obs, $dtbasedissidio);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
        return $return;
    }
    
    public function beforeSave($nrorg, $nrvinculom, $dtaltesalario, $nrocorrencia) {
        try {
            $this->validaExisteDataOcorrenciaSalario($nrorg, $nrvinculom, $dtaltesalario, $nrocorrencia);
        } catch (\Exception $e) {
            throw $e;
        }
    }  
    
    // cria novo histórico para o vínculo caso não exista
    public function afterInsert($nrorg, $nrvinculom) {
        try {
            $this->vinculoServices->verificaHistoricoCompetencia($nrorg, $nrvinculom);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    // atualiza histórico vínculo
    public function afterSave($nrorg, $nrvinculom) {
        try {
            $this->atualizaAlteSalarioHistoricoVinculo($nrorg, $nrvinculom);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaAlteSalarioHistoricoVinculo($nrorg, $nrvinculom) {
        try {
            $gpevinculoh = $this->vinculoServices->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom);

            $gpevinculoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $gpevinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
            $gpevinculoh->setDtultatu(DateUtil::getDataAtual());

            $this->entityManager->persist($gpevinculoh);
            $this->entityManager->flush($gpevinculoh);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    

    public function valorMenorSalarioMinimo($vrsalario) {
        try {
            $salariominimo = $this->valorReferenciaService->getSalarioMinimo();
            if(is_object($salariominimo)){
                $vrsalminim = $salariominimo->getVrreferencia();
                if($vrsalario < $vrsalminim){
                    return true;
                }
            }
            return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida se já existe outro salario para o vínculo com a mesma data e mesma ocorrência
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param $dtiniocupacao
     * @throws \Exception
     */
    public function validaExisteDataOcorrenciaSalario($nrorg, $nrvinculom, $dtaltesalario, $nrocorrencia) {
        try {
            $altesalarios = $this->entityManager->getRepository(Repositories::GPE_ALTESALARIO)
                ->findBy(array(
                    'nrorg' => $nrorg,
                    'nrvinculom' => $nrvinculom,
                    'dtaltesalario' => DateUtil::getDataDeString($dtaltesalario, DateUtil::FORMATO_BRASILEIRO, true),
                    'nrocorrencia' => $nrocorrencia
                ));
            if (!empty($altesalarios)) {
                throw new \Exception(Messages::HCM00000_GER_ALTESALARIO_DATA_OCOR_ALREADY_EXIST_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function nova($nrorg,$nrvinculom, $dtAlteracaoSalario, $nrOcorrencia, $nrTipoAlteracao, $vrSalario,
                         $idTipoSalario, $nrTipoModalideSalario, $observacao = null, $dtbasedissidio, $vrDiferencaReal = null, $vrPercentualReal = null,
                         $vrpercaltera = null, $vrsalariohr = null)
    {
        $alteSalario = new $this->gpe_Altesalario;
        $alteSalario->setNrvinculom($nrvinculom);
        $alteSalario->setDtaltesalario($dtAlteracaoSalario);
        $alteSalario->setDtbasedissidio($dtbasedissidio);
        $alteSalario->setNrocorrencia($nrOcorrencia);
        $alteSalario->setNrtipoaltera($nrTipoAlteracao);
        $alteSalario->setVrpercaltera($vrpercaltera);
        $alteSalario->setVrsalario($vrSalario);
        $alteSalario->setVrsalariohr($vrsalariohr);
        $alteSalario->setNrorg($nrorg);
        $alteSalario->setIdtpsalario($idTipoSalario);
        $alteSalario->setNrtpmodalidsal($nrTipoModalideSalario);
        $alteSalario->setDsobservacao($observacao);

        $alteSalario->setVrdiferencareal($vrDiferencaReal);
        $alteSalario->setVrpercreal($vrPercentualReal);

        $alteSalario->setDtinclusao(DateUtil::getDataAtual());
        $alteSalario->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $alteSalario->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $alteSalario->setDtultatu(DateUtil::getDataAtual());
        $alteSalario->setCdoperultatu($this->hcmParameters->getCdoperador());
        $alteSalario->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        
        $alteSalario->setNraltesalario(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESALARIO', 1, 12, true));
        
        $this->entityManager->persist($alteSalario);
        $this->entityManager->flush($alteSalario);
    }
    
    public function getSalarioPorCompetencia($nrorg, $nrvinculom, $dtcompetencia){
        return $this->alteSalarioRepository->getSalarioPorCompetencia($nrorg, $nrvinculom, $dtcompetencia);
    }
    
    public function getNrEscalaVinculo($nrvinculom) {
        return $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($this->hcmParameters->getNrorgTrabalho(), $nrvinculom, $this->hcmParameters->getDtCompetencia())->getNrescalatrabm();
    }

} 