<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 30/09/2016
 * Time: 13:14
 */

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use HCM\Geral\Repository\VerificacaoRepository;
use HCM\Util\FPA;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\Util;
use HCM\Util\DateUtil;
use Teknisa\Model\Gestaopessoas\Entities\GerParamverifica;
use Teknisa\Model\Gestaopessoas\Entities\GerPerfilverifica;
use Teknisa\Model\Gestaopessoas\Entities\GerVerificacao;
use Teknisa\Model\Geral\Entities\Operador;
use Teknisa\Model\Geral\Entities\Perfilacesso;

class VerificacaoService {

    protected $connection;
    protected $entityManager;
    protected $hcmParameters;

    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters, FPA $fpa, VerificacaoRepository $verificacaoRepository) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->verificacaoRepository = $verificacaoRepository;
    }

    public function ativaVerificacao($nrverificacao, $nrorg)
    {
        try {
            /** @var GerVerificacao $verificacao */
            $verificacao = $this->entityManager->getRepository(Repositories::GER_VERIFICACAO)
                ->findOneBy(array('nrverificacao'=>$nrverificacao, 'nrorg'=>$nrorg));

            $params = $this->getParametrosConsulta($verificacao->getDsconsultaverificaaux());

            if (is_array($params) && !empty($params)) {
                array_pop($params);

                foreach ($params as &$param) {
                    /** @var GerParamverifica $paramverifica */
                    $paramverifica = $this->entityManager->getRepository(Repositories::GER_PARAMVERIFICA)
                        ->findOneBy(array('nrverificacao'=>$nrverificacao, 'nmparamverifica'=>$param, 'nrorg'=>$nrorg));

                    if (!is_object($paramverifica)) {
                        throw new \Exception(str_replace('%s', $param, Messages::HCM00000_GER_VERIF_PARAM_STR_N_ENCONTR_ERROR), Messages::ERROR_MESSAGE_CODE);
                    } elseif (!$paramverifica->getVrparamverificaaux()) {
                        throw new \Exception(str_replace('%s', $param, Messages::HCM00000_GER_VERIF_PARAM_STR_S_VALOR_ERROR), Messages::ERROR_MESSAGE_CODE);
                    }
                }

                $this->verificaConsulta($verificacao->getDsconsultaverificaaux(), $params);
            } else {
                throw new \Exception(Messages::HCM00000_GER_VERIF_N_ENCONTR_PARAM_ERROR, Messages::ERROR_MESSAGE_CODE);
            }
        } catch (\Exception $e) {
            throw new \Exception("Não foi possível ativar a validação. ".$e->getMessage(), $e->getCode());
        }
    }

    public function verificaEmail($dsemailverifica)
    {
        //$valida_email = new \Doctrine_Validator_Email();

        $array_email = explode(';',$dsemailverifica);

        foreach ($array_email as $email) {
            $email = trim($email);
            if ($email) {
                //$validacao = $valida_email->validate($email);
            }

            //if (!$validacao) {
                //throw new \Exception("O email '".$email."' é inválido. Favor verificar.");
            //}
        }

        return true;
    }

    public function verificaConsulta($dsconsultaverifica, $params)
    {
        try {
            $user = $this->connection->getUsername();
            $password = $this->connection->getPassword();
            $host = $this->connection->getHost();
            $port = $this->connection->getPort();
            $serviceName = $this->connection->getParams()['dbname'];

            // connect to database
            try {
                $conn = oci_connect($user, $password, $host . ':' . $port . '/' . $serviceName);
            } catch (\Exception $e) {
                $errorConnect = oci_error();
                throw new \Exception(utf8_encode($errorConnect['message']));
            }

            // parse sql
            try {
                $stid = oci_parse($conn, $dsconsultaverifica);
            } catch (\Exception $e) {
                $errorParse = oci_error($conn);
                throw new \Exception(utf8_encode($errorParse['message']));
            }

            $valueParam = null;
            foreach ($params as $param) {
                // bind sql param
                try {
                    oci_bind_by_name($stid, $param, $valueParam);
                } catch (\Exception $e) {
                    $errorBind = oci_error($conn);
                    throw new \Exception(utf8_encode($errorBind['message']));
                }
            }

            // execute statement
            try {
                oci_execute($stid);
            } catch (\Exception $e) {
                $errorExecute = oci_error($stid);
                $message = 'Ocorreu um erro ao executar: "' . $errorExecute['sqltext'] . '". ' . utf8_encode($errorExecute['message']);
                throw new \Exception($message);
            }

            oci_free_statement($stid);
            oci_close($conn);

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function atualizaParametros($nrverificacao, $dsconsultaverifica, $nrorg)
    {
        $params = self::getParametrosConsulta($dsconsultaverifica);

        if ($params) {
            //Retira os parâmetros para as inserções.
            array_pop($params);
        }

        $this->atualizaParametrosVerificacao($nrverificacao, $params, $nrorg);
    }

    public function getParametrosConsulta($consulta)
    {
        $consulta = nl2br($consulta);
        $consulta = str_replace(array('<br />'), array(' '), $consulta);
        $consulta = rtrim($consulta, ';');
        $valida = strpbrk($consulta, ':');

        //Testa se existem parâmetros.
        if($valida) {
            $params       = array();
            $paramReplace = array();

            do{
                //Encontra o primeiro ":" e pega a string a partir daí.
                $consulta = strpbrk($consulta, ':');

                //Quebra a nova string em pedaços baseando em espaços, para pegar sua primeira posição.
                $consultaArray = explode(' ', $consulta);

                if($consultaArray[0] != '') {
                    //Possíveis caracteres que não façam parte do parâmetro.
                    if(strstr($consultaArray[0], '(') || strstr($consultaArray[0], ')') || strstr($consultaArray[0], ',') || strstr($consultaArray[0], '|')) {
                        $consultaArray[0] = str_replace(array('(', ')', ',', '|'), array(' ', ' ', ' ', ' '), $consultaArray[0]);
                        $consultaArray = explode(' ', $consultaArray[0]);
                    }
                    //Retira os caracteres não necessários do parâmetro, que é a primeira posição do array criado.
                    $param    = ltrim($consultaArray[0], ':');
                    $param    = rtrim($param);
                    $params[] = $param;
                    $paramReplace[] = rtrim($consultaArray[0]);
                }

                //Pega o resto da string, do espaço dpois do parâmetro em questão em diante para pegar os próximos.
                $consulta = strpbrk($consulta, ' ');
            } while($consulta);

            //Retira posições repetidas.
            $params = array_unique($params);
            $paramReplace = array_unique($paramReplace);

            $params[] = $paramReplace;
        } else {
            $params = false;
        }

        return $params;
    }

    public function atualizaParametrosVerificacao($nrverificacao, $array_params, $nrorg)
    {
        try {
            // Insere parâmetros novos
            foreach ($array_params as &$param) {

                /** @var GerParamverifica $paramverifica */
                $paramverifica = $this->entityManager->getRepository(Repositories::GER_PARAMVERIFICA)
                    ->findOneBy(array('nrverificacao'=>$nrverificacao, 'nmparamverifica'=>$param, 'nrorg'=>$nrorg));

                if (!is_object($paramverifica)) {
                    $this->novoParametro($nrverificacao, $param, null);
                }
            }

            // Exclui parâmetros retirados que não estão na query
            if (empty($array_params)) {
                $where =
                    "nrverificacao = $nrverificacao and nrorg = $nrorg";
            } else {
                $params = "'" . implode("', '", $array_params) . "'";
                $where =
                    "p.nrverificacao = $nrverificacao and p.nmparamverifica not in ($params) and p.nrorg = $nrorg";
            }
            //$paramverifica = GerParamverifica::getObjectsByWhere($where);
            $querybuilder = $this->entityManager->createQueryBuilder();
            $querybuilder
                ->select('p')
                ->from(Repositories::GER_PARAMVERIFICA, 'p')
                ->where($where);
            $paramverifica = $querybuilder->getQuery()->getResult();

            /** @var GerParamverifica $param */
            foreach ($paramverifica as $param) {
                $param->delete();
                $this->entityManager->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function novaVerificacao($nrverificacao, $dsverificacao, $nrtipoverifica, $dsemailverifica, $msgverificacao, $dsconsultaverifica)
    {
        try {
            $verificacao = new GerVerificacao();

            $verificacao->setNrverificacao($nrverificacao);
            $verificacao->setDsverificacao($dsverificacao);
            $verificacao->setNrtipoverifica($nrtipoverifica);
            $verificacao->setDsemailverifica($dsemailverifica);
            $verificacao->setMsgverificacao($msgverificacao);
            $verificacao->setDsconsultaverificaaux($dsconsultaverifica);
            $verificacao->setIdativo('N');
            $verificacao->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $verificacao->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $verificacao->setDtinclusao(DateUtil::getDataAtual());
            $verificacao->setCdoperinclusao($this->hcmParameters->getCdOperador());
            $verificacao->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $verificacao->setDtultatu(DateUtil::getDataAtual());
            $verificacao->setCdoperultatu($this->hcmParameters->getCdOperador());
            $verificacao->persist();
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function novoParametro($nrverificacao, $nmparamverifica, $vrparamverifica)
    {
        try {
            $param = new GerParamverifica();

            $param->setNrverificacao($nrverificacao);
            $param->setNrparamverifica(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GER_PARAMVERIFICA', 1, 22, true, false));
            $param->setNmparamverifica($nmparamverifica);
            $param->setVrparamverificaaux($vrparamverifica);
            $param->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $param->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $param->setDtinclusao(DateUtil::getDataAtual());
            $param->setCdoperinclusao($this->hcmParameters->getCdOperador());
            $param->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $param->setDtultatu(DateUtil::getDataAtual());
            $param->setCdoperultatu($this->hcmParameters->getCdOperador());
            $param->persist();
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaVerificacaoPossuiPerfil($nrorg, $nrverificacao, $nrperfilacesso)
    {
        try {
            /** @var GerPerfilverifica $paramverifica */
            $perfilverifica = $this->entityManager->getRepository(Repositories::GER_PERFILVERIFICA)
                ->findOneBy(array('nrperfilacesso'=>$nrperfilacesso, 'nrverificacao'=>$nrverificacao, 'nrorg'=>$nrorg));

            if (is_object($perfilverifica)) {
                return true;
            }

            return false;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function deleteParamsVerificacao($nrorg, $nrverificacao)
    {
        try {
            $this->connection->beginTransaction();
            $params = $this->entityManager->getRepository(Repositories::GER_PARAMVERIFICA)
                ->findBy(array('nrverificacao'=>$nrverificacao, 'nrorg'=>$nrorg));
            if (!empty($params)) {
                /** @var GerParamverifica $paramverifica */
                foreach ($params as $paramverifica) {
                    $paramverifica->delete();
                }
                $this->entityManager->flush();
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function deletePerfisVerificacao($nrorg, $nrverificacao)
    {
        try {
            $this->connection->beginTransaction();
            $perfis = $this->entityManager->getRepository(Repositories::GER_PERFILVERIFICA)
                ->findBy(array('nrverificacao'=>$nrverificacao, 'nrorg'=>$nrorg));
            if (!empty($perfis)) {
                /** @var GerPerfilverifica $perfilverifica */
                foreach ($perfis as $perfilverifica) {
                    $perfilverifica->delete();
                }
                $this->entityManager->flush();
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

    public function getVerificacoes($nrorg, $nrorgPadrao, $dtcompetencia, $cdoperador, $exibeVerifica, $agrupaVerifica, $nrVerificacao, $msgVerificacao)
    {
        try {
            $results = array();

            // Get Perfil Acesso Operador
            /** @var Operador $operador */
            $operador = $this->entityManager->getRepository(Repositories::OPERADOR)
                ->findOneBy(array('cdoperador' => $cdoperador, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));
            /** @var Perfilacesso $perfacessparc */
            $perfacessparc = $this->entityManager->getRepository(Repositories::PERFACESSPARC)
                ->findOneBy(array('nrparcnegocio' => $operador->getNrparcnegocio(), 'nrorg' => $nrorg));
            $nrperfilacesso = ($perfacessparc ? $perfacessparc->getNrperfilacesso() : 0);

            $verificacoes = $this->verificacaoRepository->retornaVerificacoesPorNrorgAndNrorgpadraoAndPerfilacesso($nrorg, $nrorgPadrao, $nrperfilacesso);

            /** @var GerVerificacao $verificacao */
            foreach ($verificacoes as $verificacao) {
                // Filtro por Verificação
                if (!is_null($nrVerificacao) && $nrVerificacao != $verificacao->getNrverificacao()) {
                    continue;
                }
                // Filtro por Mensagem
                if (!is_null($msgVerificacao) && strpos($verificacao->getMsgverificacao(), $msgVerificacao) === false) {
                    continue;
                }
                // Executa somente as verificações conforme o parametro passado ou todas caso não tenha sido informado
                if (!is_null($exibeVerifica)) {
                    $tipoverifica = $this->entityManager->getRepository(Repositories::GER_TIPOVERIFICACAO)->find($verificacao->getNrtipoverifica());
                    if ($tipoverifica->getIdexibeverifica() != $exibeVerifica) {
                        continue;
                    }
                }

                // Executa verificação
                $params = array();

                $parametros = $this->entityManager->getRepository(Repositories::GER_PARAMVERIFICA)
                    ->findBy(array('nrverificacao' => $verificacao->getNrverificacao(), 'nrorg' => $verificacao->getNrorg()));

                /** @var GerParamverifica $paramverifica */
                foreach ($parametros as $paramverifica) {

                    $param_value = $paramverifica->getVrparamverificaaux();

                    if ((strpos($param_value, '@') === 0) && (strrpos($param_value, '@') === strlen($param_value) - 1)) {
                        $param_value = Util::replaceToken($param_value);
                    }

                    $params[$paramverifica->getNmparamverifica()] = $param_value;
                }

                try {
                    $resultado = $this->verificacaoRepository->executaConsulta(Util::replaceToken($verificacao->getDsconsultaverificaaux()), $params);

                    // Coloca mensagens de validação na variável $results
                    $results = array_merge($results, $this->trataRetornoConsulta($verificacao->getMsgverificacao(), $resultado, $agrupaVerifica));

                } catch (\Exception $e) {
                    if ($agrupaVerifica === 'S') {
                        $results[] = array('VERIFICACAO' => "Ocorreu um erro na verificação '" . $verificacao->getNrverificacao() . "-" . $verificacao->getDsverificacao() . "' [" . $e->getMessage() . "]", 'QTDVERIFICACAO' => 0);
                    } else {
                        $results[] = array('MSGVERIFICACAO' => "Ocorreu um erro na verificação '" . $verificacao->getNrverificacao() . "-" . $verificacao->getDsverificacao() . "' [" . $e->getMessage() . "]");
                    }
                    continue;
                }
            }

            if (!empty($results)) {
                if ($agrupaVerifica === 'S') {
                    usort($results, function ($a, $b) {
                        return $b['QTDVERIFICACAO'] - $a['QTDVERIFICACAO'];
                    });
                } else {
                    usort($results, function ($a, $b) {
                        return strcmp($a['MSGVERIFICACAO'], $b['MSGVERIFICACAO']);
                    });
                }
            }
            return $results;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function trataRetornoConsulta($msgVerificacao, $resultado, $agrupaVerifica)
    {
        $results = array();

        // Coloca mensagens de validação na variável $results
        if (!empty($resultado)) {

            if ($agrupaVerifica === 'S') {
                $results[] = array('VERIFICACAO' => $msgVerificacao, 'QTDVERIFICACAO' => count($resultado));
            } else {
                // Percorre as linhas retornadas
                foreach ($resultado as $registro) {
                    $msgResult = "";
                    $prefix = "";

                    // Concatena os campos do registro e seus valores
                    foreach ($registro as $campo => $valor) {
                        $msgResult .= $prefix . $campo.':'.$valor;
                        $prefix = ", ";
                    }

                    $results[] = array('MSGVERIFICACAO' => $msgVerificacao . " [" . $msgResult . "]");
                }
            }
        }

        return $results;
    }
}